% SDE Drift Modeling using GPs: Bouc-Wen Example
%
% Assumes that the files for the Bouc-Wen benchmark are in the folder
% 'BoucWenFiles'. The files can be downloaded from 
% http://nonlinearbenchmark.org/.
% 
% 2017-12-19 -- Roland Hostettler <roland.hostettler@aalto.fi>

% Housekeeping
clear variables;
addpath BoucWenFiles lib;

%% Parameters
% Model parameters
J = 5;              % No of basis functions
gamma = 5;          % Fourier basis function parameter
R = 1e-9;           % Measurement noise variance

% Initial parameter guesses
theta0 = [
    log(1e-4);              % Qx (1)
    log([10, 10, 0.1]).';   % ellx (3) (x2<-x1, x2<-x1, x2<-u)
    log(100);               % ellt (1)
    log(0.001);             % sigma2t (1)
];

% Identification experiment parameters
fs = 750;           % working sampling frequency.
N = 4096;           % number of points per excitation period.
P = 1;              % number of excitation periods (5)

% Type of validation, one of 'multisine' or 'sinesweep'
% validation = 'sinesweep';
validation = 'multisine';

% Drift model
model = @(theta) gp_drift_model_bouc_wen(theta, gamma, J);

%% Bouc-Wen simulation
[y, u, t] = simulate_bouc_wen(N, P, [], fs);

% Normalize the data such that 99.6% of everything is within [-1,1]
mu = mean(u);
su = 3*std(u);
un = (u-mu)/su;
my = mean(y);
sy = 3*std(y);
yn = (y-my)/sy;
Ts = 1;             % Normalized sampling time.

%% Parameter estimation
tic;
theta_hat = fminunc(@(theta) -calculate_loglikelihood(yn, un, theta, model, Ts, R), theta0);
toc;
[~, mp, m] = calculate_loglikelihood(yn, un, theta_hat, model, Ts, R);

% Training set prediction
ytrain_hat = mp(1, :)*sy+my;
etrain = rms(ytrain_hat-y);
fprintf('Training RMSE is: %fe-5\n', etrain*1e5);

%% Validation
if strcmp(validation, 'multisine')
    load('BoucWenFiles/Test signals/Validation signals/uval_multisine.mat');
    load('BoucWenFiles/Test signals/Validation signals/yval_multisine.mat');
    utest = uval_multisine;
    ytest = yval_multisine;
elseif strcmp(validation, 'sinesweep')
    load('BoucWenFiles/Test signals/Validation signals/uval_sinesweep.mat');
    load('BoucWenFiles/Test signals/Validation signals/yval_sinesweep.mat');
    utest = uval_sinesweep;
    ytest = yval_sinesweep;
else
    error('Validation dataset not found.');
end

% Normalize (with training set normalization factors)
utn = (utest-mu)/su;
ytn = (ytest-my)/sy;

% Estimate, un-normalize and calculate test RMS error
[~, mptest, mtest] = calculate_loglikelihood(ytn, utn, theta_hat, model, Ts);
ytest_hat = sy*mptest(1, :)+my;
etest = rms(ytest_hat-ytest);

%% Naive AR(1) Predictor
% y[n|n-1] = y[n-1]
yar_hat = [0, ytest(1:end-1)];
ear_hat = rms(yar_hat-ytest);

%% Comparisons
if strcmp(validation, 'multisine')
    rmses = {
        ear_hat, 'AR(1)';
        1.126e-5, 'BLA';        % Results from nonlinearbenchmark.org
        0.895e-5, 'Volt';
        etest, 'GP';
    };
else
    rmses = {
        ear_hat, 'AR(1)';
        0.698e-5, 'BLA';        % Results from nonlinearbenchmark.org
        0.347e-5, 'Volt';
        etest, 'GP';
    };
end

%% Plots & error figures
fprintf('Validation Results\n');
fprintf('==================\n');
fprintf('Dataset: %s\n', validation);
fprintf('Model\tRMSE\t\tR2\n');
for i = 1:length(rmses)
    fprintf('%s\t%fe-5\t%f\n', rmses{i, 2}, rmses{i, 1}*1e5, 1-rmses{i, 1}/rms(ytest-mean(ytest)));
end

figure(1); clf();
plot(y); hold on;
plot(ytrain_hat);

figure(2); clf();
plot(ytest); hold on;
plot(ytest_hat);
