% GP SDE Drift Model: Cascaded Tanks Example
%
% Assumes that the files for the cascaded tanks benchmark are in the folder
% 'CascadedTanksFiles'. The files can be downloaded from 
% http://nonlinearbenchmark.org/.
% 
% 2017-12-16 -- Roland Hostettler <roland.hostettler@aalto.fi>

clear variables;
addpath lib;

%% Parameters
lambda = 10;        % Input and outputs are limited to the range 0...10
J = 5;
Ts = 4;

% Initial parameter guess
theta0 = [
	log([0.01, 0.01]).';    % Qx (2)
    log([3, 3, 3, 3]).';    % ellx (4)
    log(1000);              % ellt (1)
    log(0.1);               % sigma2t (1)
];

% Drift model
model = @(theta) gp_drift_model_tanks(theta, lambda, J);

%% Load data
load('CascadedTanksFiles/dataBenchmark.mat');
N = length(uEst);
t = (1:N)*Ts;

%% Parameter estimation
tic;
theta_hat = fminunc(@(theta) -calculate_loglikelihood(yEst.', uEst.', theta, model, Ts), theta0);
toc;
[~, mp] = calculate_loglikelihood(yEst.', uEst.', theta_hat, model, Ts);
yTrainhat = mp(1, :).';
etrain = rms(yTrainhat-yEst);

%% Validation
[~, mp] = calculate_loglikelihood(yVal.', uVal.', theta_hat, model, Ts);
yValhat = mp(1, :).';
etest = rms(yValhat-yVal);

%% Naive AR(1) Predictor
% y[n|n-1] = y[n-1]
yar_hat = [0; yVal(1:end-1)];
ear_hat = rms(yar_hat-yVal);

%%
rmses = {
    ear_hat, 'AR(1)';
    55.6e-3, 'BLA';
    19.4e-3, 'Volt';
    etest, 'GP';
};

%% Plots & error figures
% Error figures
fprintf('Validation Results\n');
fprintf('==================\n');
fprintf('Model\tRMSE\t\tR2\n');
for i = 1:length(rmses)
    fprintf('%s\t%fe-3\t%f\n', rmses{i, 2}, rmses{i, 1}*1e3, 1-rmses{i, 1}/rms(yVal-mean(yVal)));
end

% Plots
figure(1); clf();
subplot(211);
plot(t, uEst);
subplot(212);
plot(t, yEst); hold on;
plot(t, yTrainhat);

figure(2); clf();
subplot(211);
plot(t, uVal);
subplot(212);
plot(t, yVal); hold on;
plot(t, yValhat);
