function [lpy, mps, ms] = calculate_loglikelihood(y, u, theta, model, Ts, R)
% EKF Likelihood for Gaussian process drift model
%
% USAGE
%   lpy = calculate_loglikelihood(y, u, theta, model, Ts, R)
%   [lpy, mp, m] = calculate_loglikelihood(y, u, theta, model, Ts, R)
%
% DESCRIPTION
%   Log-likelihood for models of the form
%
%       dx = f(x, u, t) dt + dw,
%       y[n] = G x(t[n]),
%
%   calculated using Euler-Maruyama discretization and an extended Kalman
%   filter.
%
% PARAMETERS
%   y       Measurement signal.
%   u       Input signal.
%   theta   Model parameters.
%   model   Model constructor.
%   Ts      Sampling time (default: 1).
%   R       Measurement noise covariance (default: 1e-12).
%
% RETURNS
%   lpy     Log-likelihood.
%   mp      Predicted state mean m[n|n-1].
%   m       Filtered state mean m[n|n]
%
% AUTOR
%   2017-12-23 -- Roland Hostettler <roland.hostettler@aalto.fi>

    %% Defaults
    narginchk(4, 6);
    if nargin < 5 || isempty(Ts)
        Ts = 1;
    end
    if nargin < 6 || isempty(R)
        R = 1e-12;
    end

    %% Model
    % Drift model and Euler-Maruyama discretization
    [fc, Fcx, Qc, m0, P0] = model(theta);
    Nx = size(m0, 1);
    INx = eye(Nx);
    f = @(x, u) x + fc(x, u)*Ts;
    Fx = @(x, u) INx + Fcx(x, u)*Ts;
    Q = Qc*Ts;
    
    % Meausrement model
    G = zeros(1, Nx);
    G(1, 1) = 1;
    
    %% Initialization
    % Initialization
    N = length(y);
    lpy = 0;
    m = m0;
    P = P0;
    
    % Preallocate for speed
    ms = zeros(Nx, N);
    mps = zeros(Nx, N);

    %% Process data
    for n = 1:N
        % Prediction
        mp = f(m, u(n));
        Pp = Fx(m, u(n))*P*Fx(m, u(n))' + Q;
        
        % Measurement update
        yp = G*mp;
        S = G*Pp*G' + R;
        K = Pp*G'/S;
        m = mp + K*(y(:, n) - yp);
        P = Pp - K*S*K';

        % Likelihood
        lpy = lpy - 1/2*log(2*pi*S) - 1/2*(y(:, n)-yp).^2./S;
        
        % Store
        mps(:, n) = mp;
        ms(:, n) = m;
    end
end
