function [Psi, dPsidx, sigma2j] = fourierbasis(gamma, S, J)
% Fourier basis function expansion
%
% USAGE
%   [Psi, dPsidx, sigma2j] = fbe(gamma, J, S)
%
% DESCRIPTION
%   Calculates the Fourier basis expansion of a covariance function for the 
%   interval [-gamma,gamma] such that
%
%       f = Psi*alpha,
%       alpha ~ N(0, sigma2j).
%
% PARAMETERS
%   gamma       Interval for expansion.
%   S(w)        Power spectral density function.
%   J           Number of basis functions (default: 10).
%
% RETURNS
%   Psi(x)      Vector of basis functions (each column corresponds to a 
%               basis function).
%   dPsidx(x)   Derivative of the basis functions with respect to x.
%   sigma2j     Variances of the individual basis functions.
%
% AUTHORS
%   2017-11-29 -- Roland Hostettler <roland.hostettler@aalto.fi>

% TODO:
%   * Extend to more than 2 dimensions.

    %% Defaults
    narginchk(2, 3);
    sigma2j = [];
    if nargin < 3 || isempty(J)
        J = 10;
    end
    
    %% Basis functions and their derivatives
    switch size(S, 1)
        case 1
            %% 1D BFE
            Psi = @(x) 1/sqrt(gamma)*[
                cos(2*pi*x*(0:J)/gamma), ...
                sin(2*pi*x*(1:J)/gamma)
            ];

            dPsidx = @(x) 2*pi/(gamma*sqrt(gamma))*[
                -sin(2*pi*x*(0:J)/gamma).*(ones(size(x, 1), 1)*(0:J)), ...
                 cos(2*pi*x*(1:J)/gamma).*(ones(size(x, 1), 1)*(1:J))
            ];
        
            % Variances, if needed
            if nargin >= 3 && nargout >= 3
                sigma2j = S((0:J).'*2*pi/gamma);
                kappa = gamma/sum(sigma2j);
                sigma2j = sigma2j*kappa;
                sigma2j = [
                    sigma2j;          % DC + cos
                    sigma2j(2:end);   % sin
                ];
            end
        case 2
            %% 2D BFE
            [ii, jj] = meshgrid(0:J, 0:J);
            ii = ii(:).';
            jj = jj(:).';
            Psi = @(x) 1/sqrt(gamma)^2*[ 
                cos(2*pi*(x(1)*ii + x(2)*jj)/gamma), ...
                sin(2*pi*(x(1)*ii(2:end) + x(2)*jj(2:end))/gamma)
            ];
        
            dPsidx = {
                @(x) 2*pi/(gamma*sqrt(gamma)^2)*[
                    -sin(2*pi*(x(1)*ii + x(2)*jj)/gamma).*ii, ...
                     cos(2*pi*(x(1)*ii(2:end) + x(2)*jj(2:end))/gamma).*ii(2:end)
                ];
                @(x) 2*pi/(gamma*sqrt(gamma)^2)*[
                    -sin(2*pi*(x(1)*ii + x(2)*jj)/gamma).*jj, ...
                     cos(2*pi*(x(1)*ii(2:end) + x(2)*jj(2:end))/gamma).*jj(2:end)
                 ];
             };
            sigma2ii = S{1}(ii.'*2*pi/gamma);
            sigma2jj = S{2}(jj.'*2*pi/gamma);
            sigma2j = sigma2ii.*sigma2jj;
            sigma2j = [
                    sigma2j;          % DC + cos
                    sigma2j(2:end);   % sin
            ];
            
        otherwise
            %% Higher dimensional BFE
            error('Fourier bases for N > 2 not implemented.');
    end
end
