function [f, dfdx, Q, m0, P0] = gp_drift_model_bouc_wen(theta, gamma, J)
% GP SDE Drift Model for the Bouc-Wen System
%
% USAGE
%   [f, dfdx, Q, m0, P0] = gp_drift_model_bouc_wen(theta, lambda, J)
%
% DESCRIPTION
%   Initializes a reduced rank GP drift model for the Bouc-Wen system. The
%   model is
%
%       dx(1) = x(2) dt,
%       dx(2) = f(x, u, t) dt + dB.
%
%   See [1] for details.
%
% PARAMETERS
%   theta       Model parameters. Vector of the form
%               log([Qx, ellx (x3), ellt, sigma2t])
%   gamma       gamma parameter of the Fourier eigenbasis (default: 1).
%   J           Number of basis functions (default: 10).
%
% RETURNS
%   f(x, u)     Drift function (of the augmented system).
%   dfdx(x, u)  Hessian of the (augmented) drift function.
%   Q           Diffusion matrix.
%   m0          Initial state.
%   P0          Initial covariance matrix.
%
% REFERENCES
%   [1] R. Hostettler, F. Tronarp, S. Sarkka. "Modeling the drift function
%       in stochastic differential equations using reduced rank Gaussian
%       processes." In 18th IFAC Symposium on System Identification
%       (SYSID), 2018.
% 
% AUTHOR
%   2017-12-19 -- Roland Hostettler <roland.hostettler@aalto.fi>

    %% Defaults
    narginchk(1, 3);
    if nargin < 2 || isempty(gamma)
        gamma = 1;
    end
    if nargin < 3 || isempty(J)
        J = 10;
    end
    if length(theta) ~= 6
        error('Parameter vector mismatch, check theta.');
    end
    
    %% Unpack parameters
    Qx = exp(theta(1));
    ellx = exp(theta(2:4));
    ellt = exp(theta(5));
    sigma2t = exp(theta(6));
    
    % Diffusion and initial covariance for x
    Lx = [0; 1];
    P0x = eye(2);
    
    %% State dimensions
    % Order of the dynamic system (fixed) and input dimensionality
    Nx = 2;

    % Number of coefficients per basis function expansion dimension: J is
    % the truncation order (i.e. j = 0, ..., J) and since we use a 
    % (complex) Fourier basis, we actually get 2*J+1 coefficients (1 for 
    % DC, J for cos(.), and J for sin(.)).
    Na = 2*J+1;

    %% Basis function expansion
    % Fourier basis expansion of the kernel in x
    Psi = cell(1, 2);
    dPsidx = cell(1, 2);
    sigma2j = cell(1, 2);

    S = {
        @(w) gpk_se_psd(w, ellx(1), 1);
        @(w) gpk_se_psd(w, ellx(2), 1);
    };
    [Psi{1}, dPsidx{1}, sigma2j{1}] = fourierbasis(gamma, S, J);
    
    % FBE for u
    S = @(w) gpk_se_psd(w, ellx(3), 1);
    [Psi{2}, dPsidx{2}, sigma2j{2}] = fourierbasis(gamma, S, J);
    sigma2j = diag([sigma2j{1}; sigma2j{2}]); % Creates a diagonal matrix with stacked variances for each coefficient
    
    % Total number of states of the augmented system
    Nc = size(sigma2j, 1);
    Ntot = Nc + Nx;

    %% Spectral decomposition
    % Spectral decomposition of the temporal kernel (Matern with nu = 1/2
    % corresponds to the Ornstein-Uhlenbeck kernel).
    [Fa, La, Ha, Qa, P0a] = gpk_matern_ss(ellt, sigma2t, 1/2);
    
    % All coefficients' dynamics
    Fa = kron(eye(Nc), Fa);
    La = kron(eye(Nc), La);
    Ha = kron(eye(Nc), Ha);
    Qa = sigma2j*Qa;
    P0a = kron(eye(Nc), P0a);
    
    %% Complete model
    % Drift functions
    H = [zeros(Nc, Nx), Ha];
    fx = @(x, u) [
        x(2);
        [Psi{1}(x(1:2)), Psi{2}(u)]*H*x;
    ];
    dfxdx = @(x, u) [
        % x1, x2, coeffs
        0, 1, zeros(1, Nc); ...
        [dPsidx{1}{1}(x(1:2)), zeros(1, Na)]*H*x, ...     % dfx(2)/dx(1)
            [dPsidx{1}{2}(x(1:2)), zeros(1, Na)]*H*x, ... % dfx(2)/dx(2)
            Psi{1}(x(1:2)), ...
            Psi{2}(u);
    ];
    
    % Drift and Hessian of the coefficients
    F = [zeros(Nc, Nx), Fa];
    fa = @(x, u) F*x;
    dfadx = @(x, u) F;

    % Complete augmented system
    f = @(x, u) [
        fx(x, u);
        fa(x, u);
    ];
    dfdx = @(x, u) [
        dfxdx(x, u);
        dfadx(x, u);
    ];
    
    % Diffusion matrix
    Q = blkdiag(Lx*Qx*Lx', La*Qa*La');

    % Initial state
    m0 = zeros(Ntot, 1);
    P0 = blkdiag(P0x, P0a);
end
