function [f, dfdx, Q, m0, P0] = gp_drift_model_tanks(theta, gamma, J)
% GP SDE Drift Model for the Cascaded Tanks System
% 
% USAGE
%   [f, dfdx, Q, m0, P0] = gp_drift_model_tanks(theta, lambda, J)
%
% DESCRIPTION
%   Initializes a reduced rank GP drift model for the cascaded tanks
%   system. The model is
%
%       dx(1) = f1(x(1), u, t) dt + dB(1),
%       dx(2) = f(x(1), x(2), t) dt + dB(2).
%
%   See [1] for details.
%
% PARAMETERS
%   theta       Model parameters. Vector of the form
%               log([Qx (x2), ellx (x4), ellt, sigma2t])
%   gamma       gamma parameter of the Fourier eigenbasis (default: 1).
%   J           Number of basis functions (default: 10).
%
% RETURNS
%   f(x, u)     Drift function (of the augmented system).
%   dfdx(x, u)  Hessian of the (augmented) drift function.
%   Q           Diffusion matrix.
%   m0          Initial state.
%   P0          Initial covariance matrix.
% 
% REFERENCES
%   [1] R. Hostettler, F. Tronarp, S. Sarkka. "Modeling the drift function
%       in stochastic differential equations using reduced rank Gaussian
%       processes." In 18th IFAC Symposium on System Identification
%       (SYSID), 2018.
% 
% AUTHOR
%   2017-12-16 -- Roland Hostettler <roland.hostettler@aalto.fi>

    %% Defaults
    narginchk(1, 3);
    if nargin < 2 || isempty(gamma)
        gamma = 1;
    end
    if nargin < 3 || isempty(J)
        J = 10;
    end

    %% Unpack parameters
    Qx = diag(exp(theta(1:2)));
    ellx = exp(theta(3:6));
    ellt = exp(theta(7));
    sigma2t = exp(theta(8));

    % Diffusion and initial state
    m0x = 5*ones(2, 1);
    Lx = eye(2);
    P0x = eye(2);
    
    %% State dimensions
    % Order of the dynamic system (fixed) and input dimensionality
    Nx = 2;
    Nu = 1;

    % Number of coefficients per basis function: J is the truncation order
    % (i.e. j = 0, ..., J) and since we use a (complex) Fourier basis, we 
    % actually get 2*J+1 coefficients (1 for DC, J for cos(.), and J for
    % sin(.)).
    Na = 2*(J+1)^2-1;
    
    % Total number of coefficients: #States*(#States+#Inputs)*#Coefficients
    Nc = 2*Na;

    % Total number of states of the augmented system
    Ntot = Nc + Nx;
    
    %% Sanity checks
    % ellx
    if length(ellx) == 1
        ellx = ellx*ones(Nx, Nx + Nu);
    elseif length(ellx) ~= Nx*(Nx+Nu-1)
        error('ellx size mismatch.');
    else
        ellx = reshape(ellx(:), Nx+Nu-1, Nx).';
    end
    
    %% Basis function expansion
    % Fourier basis expansion of the kernel in x
    Psi = cell(Nx, 1);
    dPsidx = cell(Nx, 1);
    sigma2j = cell(Nx, 1);
    for i = 1:Nx
        % Note: j and i are switched for sigma so that we later on can
        % concatenate them easily into a column vector.
        S = {
            @(w) gpk_se_psd(w, ellx(i, 1), 1);
            @(w) gpk_se_psd(w, ellx(i, 2), 1);
        };
        [Psi{i}, dPsidx{i}, sigma2j{i}] = fourierbasis(gamma, S, J);
    end
    sigma2j = diag(cat(1, sigma2j{:})); % Creates a diagonal matrix with stacked variances for each coefficient
        
    %% Spectral decomposition
    % Spectral decomposition of the temporal kernel (Matern with nu = 1/2
    % corresponds to the Ornstein-Uhlenbeck kernel).
    [Fa, La, Ha, Qa, P0a] = gpk_matern_ss(ellt, sigma2t, 1/2);
    
    % All coefficients' dynamics
    Fa = kron(eye(Nc), Fa);
    La = kron(eye(Nc), La);
    Ha = kron(eye(Nc), Ha);
    Qa = sigma2j*Qa;
    P0a = kron(eye(Nc), P0a);
    
    %% Complete model
    % Drift functions
    H = [zeros(Nc, Nx), Ha];
    fx = @(x, u) [
        Psi{1}([x(1); x(2)]),      zeros(1, Na);  % Lower tank
                zeros(1, Na), Psi{2}([x(1); u]);  % Upper tank
    ]*H*x;
    dfxdx = @(x, u) [
        [dPsidx{1}{1}([x(1); x(2)]), zeros(1, Na)]*H*x, ...     % dfx(1)/dx(1)
            [dPsidx{1}{2}([x(1); x(2)]), zeros(1, Na)]*H*x, ... % dfx(1)/dx(2)
            Psi{1}([x(1); x(2)]), ...
            zeros(1, Na);
        [zeros(1, Na), dPsidx{2}{1}([x(1); u])]*H*x, ...     % dfx(2)/dx(1)
            [zeros(1, Na), dPsidx{2}{1}([x(1); u])]*H*x, ... % dfx(2)/dx(2)
            zeros(1, Na), ...
            Psi{2}([x(1); u]);
    ];
    
    % Drift and Hessian of the coefficients
    F = [zeros(Nc, Nx), Fa];
    fa = @(x, u) F*x;
    dfadx = @(x, u) F;

    % Complete augmented system
    f = @(x, u) [
        fx(x, u);
        fa(x, u);
    ];
    dfdx = @(x, u) [
        dfxdx(x, u);
        dfadx(x, u);
    ];
    
    % Diffusion matrix
    Q = blkdiag(Lx*Qx*Lx', La*Qa*La');

    % Initial state
    m0 = [m0x; zeros(Ntot-Nx, 1)];
    P0 = blkdiag(P0x, P0a);
end
