function [y, u, t] = simulate_bouc_wen(N, P, A, fs, upsamp)
% Simulate the Bouc-Wen System with Multisine Input
%
% USAGE
%   [y, u, t] = simulate_bouc_wen(N, P)
%   [y, u, t] = simulate_bouc_wen(N, P, A, fs, upsamp)
%
% DESCRIPTION
%   Simulate the Bouc-Wen system, a nonlinear oscillator with hysteresis
%   and exogeneous input. The exogeneous input is a random phase multisine 
%   excitation aimed for system identification.
%
%   This function is based on the code by J.P. Noel [1].
%
% PARAMETERS
%   N       Number of spectral lines.
%   P       Number of periods.
%   A       Excitation amplitude (default: 50).
%   fs      Sampling frequency (defualt: 750 Hz).
%   upsamp  Upsampling factor to ensure integration accuracy (default: 20).
%
% RETURNS
%   y       Output signal.
%   u       Input signal.
%   t       Time vector.
%
% REFERENCES
%   [1] Schoukens, M. and Noel, J.P. (2017). Three benchmarks addressing 
%       open challenges in nonlinear system identification. In 20th IFAC 
%       World Congress, 446-451.
%
% AUTHOR
%   2017-12-12 -- Roland Hostettler <roland.hostettler@aalto.fi>

    %% Defaults
    narginchk(2, 5);
    if nargin < 3 || isempty(A)
        A = 50;
    end
    if nargin < 4 || isempty(fs)
        fs = 750;
    end
    if nargin < 5 || isempty(upsamp)
        upsamp = 20;
    end
    
    %% Simulation
    fsint = fs*upsamp;      % integration sampling frequency.
    h = 1/fsint;            % integration time step.

    % Excitation signal design. 
    Nint = N*upsamp;        % number of points per period during integration.
    fmin = 5;               % excitation bandwidth.
    fmax = 150;

    U = zeros(Nint,1);      % definition of the multisine excitation.
    fres = fsint/Nint;
    exclines = 1:ceil(fmax/fres);
    exclines(exclines < floor(fmin/fres)) = [];

    U(exclines+1) = exp(complex(0,2*pi*rand(size(exclines))));
    u = 2*real(ifft(U));
    u = A*u/std(u);
    u = repmat(u,[P+1 1]);  % extra period to avoid edge effects during low-pass filtering

    % Simulation
    y0 = 0;                 % initial conditions.
    dy0 = 0;
    z0 = 0;
    y = BoucWen_NewmarkIntegration(h, u, y0, dy0, z0);

    % Low-pass filtering and downsampling.
    drate = factor(upsamp);        % prime factor decomposition.
    for k = 1:length(drate),
        y = decimate(y,drate(k),'fir');    
    end
    u = downsample(u,upsamp);

    % Removal of the last simulated period to eliminate the edge effects due to the low-pass filter.
    y = y(1:P*N);
    u = u(1:P*N,:).';
    t = (1:P*N)/fs;
end
